'' =================================================================================================
''
''   File....... jm_mcp4922_tryout.spin { *** NOT TESTED *** }
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2017 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 16 NOV 2017
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us

  TRM_BAUD = 115_200                                            ' terminal baud rate
  

con { io pins }                                                  
                                                                 
  RX1     = 31  { I }                                           ' serial / programming
  TX1     = 30  { O }                                            
                                                                 
  SDA1    = 29  { I/O }                                         ' i2c / eeprom
  SCL1    = 28  { I/O } 

  LDAC    =  3  { O }                                           ' mcp4922 connections 
  MOSI    =  2  { O } 
  SCK     =  1  { O } 
  CS      =  0  { O }  
  

con

  #true, ON, OFF                                                ' digital control


obj

' main                                                          ' * master Spin cog
  time : "jm_time"                                              '   timing and delays
  io   : "jm_io"                                                '   essential io
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var                                                              
                                                                 
                                                                 
pub main | level                                                        
                                                                 
  setup

  repeat
    repeat level from $000 to $FFF
      mcp4922_set_value(%1111, level)                           ' dac a, buffered, 1x
      time.pause_us(500)
       
    repeat level from $FFF to $000
      mcp4922_set_value(%1111, level)
      time.pause_us(500)  

                                                                 
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)
                                                                 
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal

  mcp4922_start(CS, SCK, MOSI, LDAC)                            ' start MCP4922  
                                                                 

con

  { --------------------------------- }
  {  P A S M   D E V E L O P M E N T  }
  { --------------------------------- }


var

  long  cog

  long  dacval

  
pub mcp4922_start(cspin, sckpin, mosipin, ldacpin)

  mcp4922_stop

  dacval.byte[0] := cspin
  dacval.byte[1] := sckpin
  dacval.byte[2] := mosipin
  dacval.byte[3] := ldacpin

  cog := cognew(@mcp4922, @dacval)

  return cog  


pub mcp4922_stop

  if (cog)
    cogstop(cog - 1)
    cog := 0

  dacval := 0


pub mcp4922_set_value(cfg, value) | work

  dacval := (1 << 31) | ((cfg & $0F) << 12) | (value & $0FFF)


pub mcp4922_busy

  return (dacval <> 0)


dat { mcp4922 driver }

                        org     0

mcp4922                 mov     t1, par                         ' get address of parameter
                        rdlong  t1, t1                          ' read pins
                        mov     t2, t1                          ' make a copy

                        and     t1, #$1F                        ' isolate cs pin
                        mov     csmask, #1
                        shl     csmask, t1                      ' convert to mask
                        or      outa, csmask                    ' make pin an output/high                   
                        or      dira, csmask

                        mov     t1, t2                          ' restore t1 (pins)
                        shr     t1, #8                          ' move sck pin to byte 0   
                        
                        and     t1, #$1F       
                        mov     sckmask, #1
                        shl     sckmask, t1    
                        andn    outa, sckmask  
                        or      dira, sckmask

                        mov     t1, t2
                        shr     t1, #16        
                        
                        and     t1, #$1F       
                        mov     mosimask, #1
                        shl     mosimask, t1   
                        andn    outa, mosimask 
                        or      dira, mosimask
                            
                        mov     t1, t2
                        shr     t1, #24        
                        
                        and     t1, #$1F       
                        mov     ldacmask, #1
                        shl     ldacmask, t1   
                        or      outa, ldacmask 
                        or      dira, ldacmask

clr_command             mov     t1, #0
                        wrlong  t1, par                         ' clear pins from dacval
                        nop
                        nop

get_command             rdlong  t1, par                 wc      ' get command
        if_z            jmp #get_command

shift_out               shl     t1, #16                         ' left-align msb
                        mov     t2, #16                         ' shift 16 bits
                        andn    outa, csmask                    ' cs low
                        
:loop                   rcl     t1, #1                  wc      ' move msb to C
                        muxc    outa, mosimask                  ' copy C to mosi
                        nop 
                        or      outa, sckmask                   ' sck high
                        nop
                        andn    outa, sckmask                   ' sck low
                        djnz    t2, #:loop                      ' update bit count, do next       

                        or      outa, csmask                    ' cs high
                        andn    outa, ldacmask                  ' ldac low
                        or      outa, ldacmask                  ' ldac high

                        jmp     #clr_command
                        
' -------------------------------------------------------------------------------------------------

csmask                  res     1                               ' mask for cs pin
sckmask                 res     1                               ' mask for sck (clock) pin
mosimask                res     1                               ' mask for mosi (data) pin
ldacmask                res     1                               ' mask for ldac (load) pin

t1                      res     1                               ' work vars
t2                      res     1

         
dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      